// Copyright (c) 2002 Cunningham & Cunningham, Inc.
// Released under the terms of the GNU General Public License version 2 or later.

package eg.music;

import fit.*;
import java.util.Date;

public class Simulator extends TimedActionFixture {


    // Simulation ///////////////////////////////

    static long time = new Date().getTime();

    public static long nextSearchComplete = 0;
    public static long nextPlayStarted = 0;
    public static long nextPlayComplete = 0;

    static long next(long bound) {
        long result = bound;
        result = sooner(result, nextSearchComplete);
        result = sooner(result, nextPlayStarted);
        result = sooner(result, nextPlayComplete);
        return result;
    }

    static long sooner (long soon, long event) {
        return event > time && event < soon ? event : soon;
    }

    static void events() {
        if (time == nextSearchComplete)     {Music.searchComplete();}
        if (time == nextPlayStarted)        {Music.playStarted();}
        if (time == nextPlayComplete)       {Music.playComplete();}
    }

    static void simulate (long future) {
        while (time < future) {
            time = next(future);
            events();
        }
    }

    static long schedule (double seconds) {
        return time + (long)(1000 * seconds);
    }

    // Actions //////////////////////////////////

    public Date time () {
        return new Date(time);
    }

    public void pause () {
        double seconds = Double.parseDouble(cells.more.text());
        simulate(schedule(seconds));
    }

    public void await () throws Exception {
        String eventName = camel("next "+cells.more.text());
        long eventTime = getClass().getField(eventName).getLong(this);
        if (eventTime > 0) {
            simulate(eventTime);
        }
    }

    public void fail () throws Exception {
        String failButton = camel("fail "+cells.more.text());
        method(failButton, 0).invoke(actor, empty);
    }

    public void enter() throws Exception {
        simulate(schedule(0.8));
        super.enter();
    }

    public void press() throws Exception {
        simulate(schedule(1.2));
        super.press();
    }

}
